/*
 * dip		A program for handling dialup IP connecions.
 *		This program handles the connections needed for dialup
 *		IP links, like SLIP or PPP.  It can handle both incoming
 *		and outgoing connections, using password security for
 *		incoming connections.  The outgoing connections use the
 *		system's dial(3) library if possible.
 *
 * Usage:	See help display below.
 *
 * Version:	@(#)main.c	3.3.7	12/13/93
 *
 * Author:      Fred N. van Kempen, <waltje@uWalt.NL.Mugnet.ORG>
 *		Copyright 1988-1993 MicroWalt Corporation
 *
 *              Slight modifications made by the infamous lilo. :)
 *
 *		This program is free software; you can redistribute it
 *		and/or  modify it under  the terms of  the GNU General
 *		Public  License as  published  by  the  Free  Software
 *		Foundation;  either  version 2 of the License, or  (at
 *		your option) any later version.
 */
#define GLOBAL
#include "dip.h"


#define VERSION	"3.3.7.lilo-3.2 (18 March 1994)"


char *Version = "@(#) dip " VERSION;


struct dip	mydip;			/* global DIP entry		*/
int		opt_b = 1;		/* run-in-background flag	*/
int		opt_n = 0;		/* open tty non-blocking flag	*/
int		opt_r = 0;		/* random reconnect wait flag	*/
int		opt_v = 0;		/* debug flag			*/


/* Catch any signals. */
static void
pre_catcher1(int sig)
{
  if (sig==SIGHUP) exit(2);
  if (sig==SIGINT) exit(3);
  if (sig==SIGTERM) exit(4);
  if (sig==SIGALRM) exit(5);
  exit(100+sig);
}


/* Kill a running DIP process. */
static void
kill_dip(void)
{
  char buff[128];
  int pid;
  FILE *fp;

  fp = fopen(mydip.pidfile, "r");
  if (fp == NULL) {
	fprintf(stderr, "DIP: cannot open %s: %s\n",
			mydip.pidfile, strerror(errno));
	return;
  }
  (void) fgets(buff, 128, fp);
  pid = atoi(buff);
  if (kill(pid, SIGTERM)) {
	fprintf(stderr, "DIP: cannot kill process %d: %s\n",
			pid, strerror(errno));
	return;
  } else printf("DIP: process %d killed.\n", pid);
  (void) fclose(fp);
  (void) unlink(mydip.pidfile);
}


/* Miscellaneous initialization; fill in the global DIP entry. */
static void
dip_init(void)
{
  int i;
  struct passwd *pw;
  struct hostent *hp;
  
  for(i = 1; i < 32; i++)
  	if (i != SIGHUP && i != SIGINT && i != SIGTERM)
  		(void) signal(i, SIG_IGN);
  (void) signal(SIGHUP, pre_catcher1);
  (void) signal(SIGINT, pre_catcher1);
  (void) signal(SIGTERM, pre_catcher1); 

  if ((pw = getpwuid(getuid())) == (struct passwd *)NULL) {
	fprintf(stderr, "You do not exist.  Go away!\n");
	exit(-1);
  }
  memset((char *) &mydip, 0, sizeof(struct dip));
  strncpy(mydip.name, pw->pw_name, 16);
  strcpy(mydip.home, "/tmp");

  if (gethostname(mydip.local, 128) < 0) {
	perror("gethostname");
	exit(-1);
  }
  if ((hp = gethostbyname(mydip.local)) == (struct hostent *)NULL) {
	herror(mydip.local);
	strcpy(mydip.local, "");
	exit(-1);
  }
  strncpy(mydip.local, hp->h_name, 128);
  memcpy((char *) &mydip.loc_ip, (char *) hp->h_addr_list[0], hp->h_length);

  strcpy(mydip.protocol, DEF_PROT);
  mydip.protonr = get_prot(mydip.protocol);
  mydip.mtu = DEF_MTU;
  mydip.shellfile[0] = '\0';
  strcpy(mydip.pidfile, _PATH_DIP_PID "dip.pid");
  mydip.timeout = 0;
}


static void
usage(void)
{
  fprintf(stderr, "Usage: dip -i [-v]\n");
  fprintf(stderr, "       diplogin [-v]\n");
  fprintf(stderr, "       dip -k [-v] [-f path]\n");
  fprintf(stderr, "       dip -t [-v] [-b] [-n] [-f path] [-c count]\n");
  fprintf(stderr, "       dip [-v] [-m mtu] [-p proto] [telno | script]"\
    "[-n] [-b] [-c count]\n");
  fprintf(stderr, "           [-f path]\n\n");
  fprintf(stderr, "Flags: -b        start protocol daemon in background\n");
  fprintf(stderr, "       -c count  set startup time limit to count\n");
  fprintf(stderr, "       -f path   for file containing pid, use \"path\"\n");
  fprintf(stderr, "       -i        use dip in dial-in server mode\n");
  fprintf(stderr, "       -k        kill the specified protocol server\n");
  fprintf(stderr, "       -n        open protocol tty non-blocking\n");
  fprintf(stderr, "       -p proto  protocol to use\n");
  fprintf(stderr, "       -s        shell script to complete protocol setup\n");
  exit(-1);
}


int
main(int argc, char *argv[])
{
  char path[128];
  FILE *fp;
  register int s;
  register char *sp;
  int opt_i, opt_k, opt_t;

  /* Setup. */
  dip_init();

  strcpy(path, "");
  opt_i = 0;
  opt_k = 0;
  opt_t = 0;

  /*
   * This is a kludge for bad "login" programs which cannot pass
   * arguments. Needless to say, this includes the SLS login(8)
   * program, and most BSD derivatives... -FvK
   */
  if ((sp = strrchr(argv[0], '/')) != NULL) *sp = '\0';
    else sp = argv[0];
  if ((*sp == '-') || !strcmp(sp, "diplogin")) opt_i = 1;

  /* Scan command line for any arguments. */
  opterr = 0;
  while ((s = getopt(argc, argv, "bc:f:ikl:m:np:rs:tvw:")) != EOF) switch(s) {

  	case 'b':
  		opt_b = 1 - opt_b;
  		break;
  	
  	case 'c':
  		mydip.timeout = atoi(optarg);
		if (mydip.timeout <= 0 || mydip.timeout > 32767) usage();
		break;

	case 'f':
		strcpy(mydip.pidfile,_PATH_DIP_PID);
		strcat(mydip.pidfile, optarg);
		strcat(mydip.pidfile, ".pid");
		break;
  		
	case 'i':
		opt_i = 1 - opt_i;
		break;

	case 'k':
		opt_k = 1 - opt_k;
		break;

	case 'm':
		mydip.mtu = atoi(optarg);
		if (mydip.mtu <= 0 || mydip.mtu > 32767) usage();
		break;

	case 'n':
		opt_n = 1 - opt_n;
		break;

	case 'p':
		strcpy(mydip.protocol, optarg);
		mydip.protonr = get_prot(mydip.protocol);
		if (mydip.protonr == 0) usage();
		break;

	case 's':
		strcpy(mydip.shellfile, optarg);
		break;

	case 't':
		opt_t = 1 - opt_t;
		break;

	case 'v':
		opt_v = 1 - opt_v;
		break;

	default:
		usage();
  }

  printf("DIP: Dialup IP Protocol Driver version %s\n", VERSION);
  printf("Written by Fred N. van Kempen, MicroWalt Corporation; ");
  printf("hacked slightly by lilo.\n\n");

  /* Are we called to kill off a DIP process? */
  if (opt_k == 1) {
	kill_dip();
	exit(0);
  }

  /* Are we going to be a dialIN server? */
  if (opt_i == 1) {
	if (optind == argc) sp = mydip.name;
	  else sp = argv[optind];
	do_login(sp, &argv[0]);
	/*NOTREACHED*/
  }

  /* Are we running in TEST mode? */
  if (opt_t == 1) {
	if (optind != argc) usage();
	do_command(stdin);
	/*NOTREACHED*/
  }

  /* No, so we are running in dialOUT mode. */
  if (optind != (argc - 1)) usage();
  strncpy(path, argv[optind], 128);
  if ((sp = strrchr(path, '/')) != (char *)NULL) sp++;
    else sp = path;
  if (strchr(sp, '.') == (char *)NULL) strcat(path, DIP_SFX);

  if ((fp = fopen(path, "r")) == (FILE *)NULL) {
	fprintf(stderr, "dip: %s: %s\n", path, strerror(errno));
	exit(-1);
  }
  (void) setbuf(fp, (char *)NULL);
  do_command(fp);

  /*NOTREACHED*/
  return(-1);
}
